# Generated by Django 2.2.17 on 2021-02-02 17:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0121_systemwiderole_add_ent_cust_field'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='enterprisecustomercatalog',
            name='sync_enterprise_catalog_query',
        ),
        migrations.RemoveField(
            model_name='historicalenterprisecustomercatalog',
            name='sync_enterprise_catalog_query',
        ),
    ]
